@@ -3,7 +3,7 @@
 #include <skalibs/textclient.h>
 #include <s6/ftrigr.h>
 
-int ftrigr_startf (ftrigr_t *a, tain_t const *deadline, tain_t *stamp)
+int ftrigr_startf (ftrigr_t *a, tain const *deadline, tain *stamp)
 {
   static char const *const cargv[2] = { FTRIGRD_PROG, 0 } ;
   static char const *const cenvp[1] = { 0 } ;
