@@ -138,7 +138,6 @@ static inline int read_downsig (void)
 static void set_down_and_ready (char const *s, unsigned int n)
 {
   status.pid = 0 ;
-  status.pgid = 0 ;
   status.flagfinishing = 0 ;
   status.flagready = 1 ;
   state = DOWN ;
@@ -163,7 +162,7 @@ static void bail (void)
 
 static void killI (void)
 {
-  if (status.pgid > 0) killpg(status.pgid, SIGINT) ;
+  killpg(status.pid, SIGINT) ;
 }
 
 static void sigint (void)
@@ -267,27 +266,21 @@ static void killr (void)
 
 static void killP (void)
 {
-  if (status.pgid > 0)
-  {
-    killpg(status.pgid, SIGSTOP) ;
-    status.flagpaused = 1 ;
-    announce() ;
-  }
+  killpg(status.pid, SIGSTOP) ;
+  status.flagpaused = 1 ;
+  announce() ;
 }
 
 static void killC (void)
 {
-  if (status.pgid > 0)
-  {
-    killpg(status.pgid, SIGCONT) ;
-    status.flagpaused = 0 ;
-    announce() ;
-  }
+  killpg(status.pid, SIGCONT) ;
+  status.flagpaused = 0 ;
+  announce() ;
 }
 
 static void killK (void)
 {
-  if (status.pgid > 0) killpg(status.pgid, SIGKILL) ;
+  killpg(status.pid, SIGKILL) ;
 }
 
 static void trystart (void)
@@ -297,21 +290,23 @@ static void trystart (void)
     [0] = { .type = CSPAWN_FA_CLOSE },
     [1] = { .type = CSPAWN_FA_MOVE },
   } ;
-  char lkfmt[UINT_FMT] ;
-  char const *cargv[8] = { S6_BINPREFIX "s6-setlock", "-d", lkfmt, "--", SLCK, "./run", servicename, 0 } ;
-  size_t orig = 5 ;
+  char const *cargv[8] ;
   int notifyp[2] = { -1, -1 } ;
   unsigned int lk = 0, notif = 0 ;
   uint16_t cspawnflags = CSPAWN_FLAGS_SELFPIPE_FINISH | CSPAWN_FLAGS_SETSID ;
+  uint8_t m = 0 ;
+  char lkfmt[UINT_FMT] ;
+
+  if (check_file("flag-newpidns")) cspawnflags |= CSPAWN_FLAGS_NEWPIDNS ;
 
   if (read_uint("lock-fd", &lk))
   {
-    if (lk > maxfd) strerr_warnw2x("invalid ", "lock-fd") ;
+    if (!lk || lk > maxfd) strerr_warnw2x("invalid ", "lock-fd") ;
     else
     {
       struct stat st ;
       int islocked ;
-      int lfd = open_write(SLCK) ;
+      int lfd = open_trunc(SLCK) ;
       if (lfd == -1)
       {
         settimeout(60) ;
@@ -342,14 +337,18 @@ static void trystart (void)
         strerr_warnw1x("another instance of the service is already running, child will block") ;
       fd_close(lfd) ;
       lkfmt[uint_fmt(lkfmt, lk)] = 0 ;
-      orig = 0 ;
+      cargv[m++] = S6_BINPREFIX "s6-setlock" ;
+      cargv[m++] = "-d" ;
+      cargv[m++] = lkfmt ;
+      cargv[m++] = "--" ;
+      cargv[m++] = SLCK ;
     }
   }
 
   if (read_uint("notification-fd", &notif))
   {
-    if (notif > maxfd) strerr_warnw2x("invalid ", "notification-fd") ;
-    if (!orig && notif == lk)
+    if (!notif || notif > maxfd) strerr_warnw2x("invalid ", "notification-fd") ;
+    if (notif == lk)
     {
       settimeout_infinite() ;
       strerr_warnwu1x("start service: notification-fd and lock-fd are the same") ;
@@ -366,12 +365,14 @@ static void trystart (void)
     fa[1].x.fd2[1] = notifyp[1] ;
   }
 
-  if (check_file("flag-newpidns")) cspawnflags |= CSPAWN_FLAGS_NEWPIDNS ;
-  status.pid = cspawn(cargv[orig], cargv + orig, (char const *const *)environ, cspawnflags, fa, notifyp[1] >= 0 ? 2 : 0) ;
+  cargv[m++] = "./run" ;
+  cargv[m++] = servicename ;
+  cargv[m++] = 0 ;
+  status.pid = cspawn(cargv[0], cargv, (char const *const *)environ, cspawnflags, fa, notifyp[1] >= 0 ? 2 : 0) ;
   if (!status.pid)
   {
     settimeout(60) ;
-    strerr_warnwu3sys("spawn ", cargv[orig], " (waiting 60 seconds)") ;
+    strerr_warnwu3sys("spawn ", cargv[0], " (waiting 60 seconds)") ;
     goto errn ;
   }
 
@@ -380,9 +381,6 @@ static void trystart (void)
     fd_close(notifyp[1]) ;
     notifyfd = notifyp[0] ;
   }
-  status.pgid = getpgid(status.pid) ;
-  if (status.pgid == -1)
-    strerr_warnwu1sys("getpgid() (process group control commands will have no effect)") ;
   settimeout_infinite() ;
   nextstart = tain_zero ;
   state = UP ;
@@ -454,7 +452,7 @@ static int uplastup_z (void)
   char fmt0[UINT_FMT] ;
   char fmt1[UINT_FMT] ;
   char fmt2[PID_FMT] ;
-  char const *cargv[6] = { "finish", fmt0, fmt1, servicename, status.pgid > 0 ? fmt2 : "-1", 0 } ;
+  char const *cargv[6] = { "finish", fmt0, fmt1, servicename, fmt2, 0 } ;
 
   status.flagpaused = 0 ;
   status.flagready = 0 ;
@@ -467,7 +465,7 @@ static int uplastup_z (void)
   }
   fmt0[uint_fmt(fmt0, WIFSIGNALED(status.wstat) ? 256 : WEXITSTATUS(status.wstat))] = 0 ;
   fmt1[uint_fmt(fmt1, WTERMSIG(status.wstat))] = 0 ;
-  if (status.pgid > 0) fmt2[pid_fmt(fmt2, status.pgid)] = 0 ;
+  fmt2[pid_fmt(fmt2, status.pid)] = 0 ;
 
   if (!read_uint("max-death-tally", &n)) n = 100 ;
   if (n > S6_MAX_DEATH_TALLY) n = S6_MAX_DEATH_TALLY ;
@@ -501,7 +499,6 @@ static int uplastup_z (void)
       tain_add_g(&deadline, &tto) ;
     else settimeout_infinite() ;
   }
-  status.pgid = getpgid(status.pid) ;
   status.flagfinishing = 1 ;
   announce() ;
   ftrigw_notifyb_nosig(S6_SUPERVISE_EVENTDIR, "d", 1) ;
@@ -794,7 +791,8 @@ int main (int argc, char const *const *argv)
     progname[proglen] = ' ' ;
     memcpy(progname + proglen + 1, argv[1], namelen + 1) ;
     PROG = progname ;
-    if (!fd_sanitize()) strerr_diefu1sys(111, "sanitize stdin and stdout") ;
+    if (!fd_sanitize())
+      strerr_diefu1sys(111, "sanitize stdin and stdout") ;
     {
       struct rlimit rl ;
       if (getrlimit(RLIMIT_NOFILE, &rl) == -1)
@@ -804,7 +802,8 @@ int main (int argc, char const *const *argv)
     x[1].fd = control_init() ;
     x[0].fd = selfpipe_init() ;
     if (x[0].fd == -1) strerr_diefu1sys(111, "init selfpipe") ;
-    if (!sig_altignore(SIGPIPE)) strerr_diefu1sys(111, "ignore SIGPIPE") ;
+    if (!sig_altignore(SIGPIPE))
+      strerr_diefu1sys(111, "ignore SIGPIPE") ;
     {
       sigset_t set ;
       sigemptyset(&set) ;
@@ -832,7 +831,9 @@ int main (int argc, char const *const *argv)
     settimeout(0) ;
     tain_copynow(&status.stamp) ;
     status.readystamp = status.stamp ;
+    status.flagpaused = 1 ;
     announce() ;
+    status.flagpaused = 0 ;
     ftrigw_notifyb_nosig(S6_SUPERVISE_EVENTDIR, "s", 1) ;
 
     while (gflags.cont)
